// IndexGen1.cpp: Implementierung der Klasse CIndexGen.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "stdafx.h"
#include "IndexGen1.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CIndexGen::CIndexGen()
{}


CIndexGen::~CIndexGen()
{}


BOOL CIndexGen::Generate(LPCTSTR lpszSourceDir, LPCTSTR lpszIndexPath)
{
	m_strSourceDir = lpszSourceDir;
	m_strIndexPath = lpszIndexPath;

	// analyze all files
	CFileFind	ff;
	BOOL			bMoreFiles = ff.FindFile(m_strSourceDir + _T("\\*.htm*"));

	while (bMoreFiles)
	{
		bMoreFiles = ff.FindNextFile();
		if (!ff.IsDirectory())
		{
			m_strCurrentFile = ff.GetFileName();
			AnalyzeFile(ff.GetFilePath());
		}
	}

	// generate index file
	GenerateIndexFile();

	return TRUE;
}


BOOL CIndexGen::AnalyzeFile(LPCTSTR lpszPath)
{
	try
	{
		CFile			file(lpszPath, CFile::modeRead|CFile::shareExclusive);
		CArchive	ar(&file, CArchive::load);

		CString	strLine;
		while (ar.ReadString(strLine))
		{
			Lexer(strLine);
		}

		ar.Close();
		file.Close();
	
		return TRUE;
	}
	catch (CException *pE)
	{
		pE->Delete();
		return FALSE;
	}
}


int CIndexGen::FindHtmlTag(const CString &strText, int nStart, CString &strTag)
{
	strTag.Empty();

	int	nTagStart = strText.Find(_T('<'), nStart);
	if (nTagStart < 0)
		return -1;

	int	nTagEnd = strText.Find(_T('>'), nTagStart);
	if (nTagEnd < 0)
		return -1;

	strTag = strText.Mid(nTagStart, nTagEnd-nTagStart+1);
	strTag.MakeUpper();

	return nTagStart;
}


void CIndexGen::Lexer(const CString &strLine)
{
	static CString	strToken;

	CString	strTag;
	int			nStart = 0;
	int			nStartNext = -1;

	while ((nStartNext = FindHtmlTag(strLine, nStart, strTag)) > -1)
	{
		if (nStartNext > nStart)
		{
			strToken+= _T(' ') + strLine.Mid(nStart, nStartNext-nStart);
			strToken.TrimLeft();
			strToken.TrimRight();
			Parser(strToken, FALSE);
		}
		else if (!strToken.IsEmpty())
			Parser(strToken, FALSE);

		strToken.Empty();
		Parser(strTag, TRUE);

		nStart = nStartNext+strTag.GetLength();
	}

	if (nStart < strLine.GetLength()-1)
	{
		strToken+= _T(' ') + strLine.Mid(nStart);
		strToken.TrimLeft();
		strToken.TrimRight();
	}
}


void CIndexGen::Parser(CString strToken, BOOL bHtmlTag)
{
	static BOOL bInRef = FALSE;
	static BOOL	bIgnore_a = FALSE;
	static BOOL bIgnore_pre = FALSE;

	if (!bHtmlTag && bInRef && !bIgnore_a && !bIgnore_pre)
	{
		if (strToken.GetLength() && strToken[0] == _T('\\'))
		{
			int	nIndex;
			if ((nIndex = strToken.Find(_T('{'))) > -1)
				// remove any arguments
				strToken = strToken.Left(nIndex);
			if ((nIndex = strToken.Find(_T('['))) > -1)
				// remove any arguments
				strToken = strToken.Left(nIndex);
			if ((nIndex = strToken.Find(_T('('))) > -1)
				// remove any arguments
				strToken = strToken.Left(nIndex);
		}

		m_aEntries.Add(CIndexEntry(strToken, m_strCurrentFile));
		bInRef = FALSE;
	}
	else if (bIgnore_a && strToken == _T("</A>"))
		bIgnore_a = FALSE;
	else if (bIgnore_pre && strToken == _T("</PRE>"))
		bIgnore_pre = FALSE;
	else if (strToken.Left(2) == _T("<A"))
	{
		bIgnore_a = TRUE;
		bInRef = FALSE;
	}
	else if (strToken.Left(4) == _T("<PRE"))
	{
		bIgnore_pre = TRUE;
		bInRef = FALSE;
	}
	else if (!bIgnore_pre && !bIgnore_a)
	{
		if (strToken == _T("<CODE>") || strToken == _T("<H1>"))
			bInRef = TRUE;
	}
}


BOOL CIndexGen::GenerateIndexFile()
{
	try
	{
		CFile			file(m_strIndexPath, CFile::modeCreate|CFile::modeWrite|CFile::shareExclusive);
		CArchive	ar(&file, CArchive::store);

		// write header
		ar.WriteString(_T("<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">\r\n"));
		ar.WriteString(_T("<HTML>\r\n"));
		ar.WriteString(_T("<HEAD>\r\n"));
		ar.WriteString(_T("<meta name=\"GENERATOR\" content=\"Microsoft&reg; HTML Help Workshop 4.1\">\r\n"));
		ar.WriteString(_T("<!-- Sitemap 1.0 -->\r\n"));
		ar.WriteString(_T("</HEAD><BODY>\r\n"));
		ar.WriteString(_T("<UL>\r\n"));

		// insert elements
		for (int i = 0; i < m_aEntries.GetSize(); i++)
		{
			CString	strFormat;

			ar.WriteString(_T("\t<LI> <OBJECT type=\"text/sitemap\">\r\n"));
			strFormat.Format(_T("\t\t<param name=\"Name\" value=\"%s\">\r\n"), m_aEntries[i].m_strTitle);
			ar.WriteString(strFormat);
			strFormat.Format(_T("\t\t<param name=\"Local\" value=\"%s\">\r\n"), m_aEntries[i].m_strFile);
			ar.WriteString(strFormat);
			ar.WriteString(_T("\t\t</OBJECT>\r\n"));
		}

		// write footer
		ar.WriteString(_T("</UL>\r\n"));
		ar.WriteString(_T("</BODY></HTML>\r\n"));

		return TRUE;
	}
	catch (CException *pE)
	{
		pE->Delete();
		return FALSE;
	}
}