/********************************************************************
*
* This file is part of the TeXnicCenter-system
*
* Copyright (C) 1999-2000 Sven Wiegand
* Copyright (C) 2000-$CurrentYear$ ToolsCenter
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* If you have further questions or if you want to support
* further TeXnicCenter development, visit the TeXnicCenter-homepage
*
*    http://www.ToolsCenter.org
*
*********************************************************************/

#include "stdafx.h"
#include "TeXnicCenter.h"
#include "CustomizePageEditor.h"
#include "configuration.h"
#include "LatexEdit.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//-------------------------------------------------------------------
// class CCustomizePageEditor 
//-------------------------------------------------------------------


BEGIN_MESSAGE_MAP(CCustomizePageEditor, CPropertyPage)
	//{{AFX_MSG_MAP(CCustomizePageEditor)
	ON_CBN_SELCHANGE(IDC_EDITOR_ELEMENT, OnSelchangeEditorElement)
	ON_BN_CLICKED(IDC_EDITOR_SELECTFONT, OnEditorSelectfont)
	ON_BN_CLICKED(IDC_EDITOR_ELEMENTCOLOR, OnElementColor)
	ON_WM_DESTROY()
	ON_CBN_SELCHANGE(IDC_WINDOW, OnSelchangeWindow)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


IMPLEMENT_DYNCREATE(CCustomizePageEditor, CPropertyPage)


CCustomizePageEditor::CCustomizePageEditor() 
: CPropertyPage(CCustomizePageEditor::IDD)
{
	// copy color array
	for( int i = 0; i < CCrystalTextView::COLORINDEX_ERRORBKGND; i++ )
		m_aColors[i] = g_configuration.m_aEditorColors[i];

	m_nEditorElement = 0;
	m_strTabWidth.Format( _T("%d"), g_configuration.m_nTabWidth );

	// initialize
	m_font.CreateFontIndirect(&g_configuration.m_fontEditor);

	//{{AFX_DATA_INIT(CCustomizePageEditor)
	//}}AFX_DATA_INIT

	// preselect a window type
	m_nWindowElement = wndEditor;
}


CCustomizePageEditor::~CCustomizePageEditor()
{
	m_font.DeleteObject();
}


void CCustomizePageEditor::UpdateFontExample()
{
	m_font.DeleteObject();
	m_font.CreateFontIndirect(&m_logfont);
	m_wndFontExample.SetFont(&m_font);
	m_wndFontExample.SetWindowText(m_logfont.lfFaceName);
}


void CCustomizePageEditor::UseWindowTypeSettings()
{
	switch (m_nWindowElement)
	{
		case wndEditor:
			memcpy(&m_logfont, &g_configuration.m_fontEditor, sizeof(m_logfont));
			break;

		case wndOutput:
			memcpy(&m_logfont, &g_configuration.m_fontOutput, sizeof(m_logfont));
			break;

		case wndNavigator:
			memcpy(&m_logfont, &g_configuration.m_fontNavigator, sizeof(m_logfont));
			break;

		default:
			ASSERT(FALSE);
	}

	UpdateControlStates();
	UpdateFontExample();
}


void CCustomizePageEditor::StoreWindowTypeSettings()
{
	switch (m_nWindowElement)
	{
		case wndEditor:
			memcpy(&g_configuration.m_fontEditor, &m_logfont, sizeof(m_logfont));
			break;

		case wndOutput:
			memcpy(&g_configuration.m_fontOutput, &m_logfont, sizeof(m_logfont));
			break;

		case wndNavigator:
			memcpy(&g_configuration.m_fontNavigator, &m_logfont, sizeof(m_logfont));
			break;

		default:
			ASSERT(FALSE);
	}
}


void CCustomizePageEditor::UpdateControlStates()
{
	m_wndEditorElement.EnableWindow(m_nWindowElement == wndEditor);
	m_wndElementColorPicker.EnableWindow(m_nWindowElement == wndEditor);
	m_wndEditorTabWidth.EnableWindow(m_nWindowElement == wndEditor);
}


void CCustomizePageEditor::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCustomizePageEditor)
	DDX_Control(pDX, IDC_EDITOR_TABWIDTH, m_wndEditorTabWidth);
	DDX_Control(pDX, IDC_EDITOR_FONT, m_wndFontExample);
	DDX_Control(pDX, IDC_EDITOR_ELEMENT, m_wndEditorElement);
	DDX_Control(pDX, IDC_EDITOR_ELEMENTCOLOR, m_wndElementColorPicker);
	DDX_CBIndex(pDX, IDC_EDITOR_ELEMENT, m_nEditorElement);
	DDX_Text(pDX, IDC_EDITOR_TABWIDTH, m_strTabWidth);
	DDX_CBIndex(pDX, IDC_WINDOW, m_nWindowElement);
	//}}AFX_DATA_MAP
}


BOOL CCustomizePageEditor::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// set item data for element combo
	m_wndEditorElement.SetItemData( 0, CCrystalTextView::COLORINDEX_SELMARGIN );
	m_wndEditorElement.SetItemData( 1, CCrystalTextView::COLORINDEX_WHITESPACE );
	m_wndEditorElement.SetItemData( 2, CCrystalTextView::COLORINDEX_BKGND );
	m_wndEditorElement.SetItemData( 3, CCrystalTextView::COLORINDEX_NORMALTEXT );
	m_wndEditorElement.SetItemData( 4, CCrystalTextView::COLORINDEX_SELBKGND );
	m_wndEditorElement.SetItemData( 5, CCrystalTextView::COLORINDEX_SELTEXT );
	m_wndEditorElement.SetItemData( 6, CCrystalTextView::COLORINDEX_KEYWORD );
	m_wndEditorElement.SetItemData( 7, CCrystalTextView::COLORINDEX_COMMENT );
	m_wndEditorElement.SetItemData( 8, CCrystalTextView::COLORINDEX_OPERATOR );
	m_wndEditorElement.SetItemData( 9, CCrystalTextView::COLORINDEX_STRING );
	m_wndEditorElement.SetItemData( 10, CCrystalTextView::COLORINDEX_NUMBER );
	m_wndEditorElement.SetItemData( 11, CCrystalTextView::COLORINDEX_PREPROCESSOR );

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// select color
	m_wndElementColorPicker.m_bEnabledInCustomizeMode = TRUE;
	m_wndElementColorPicker.EnableOtherButton(CString((LPCTSTR)STE_COLOR_OTHERS));
	OnSelchangeEditorElement();

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// use the right settings
	UseWindowTypeSettings();

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// use data
	UpdateData( FALSE );

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX-Eigenschaftenseiten sollten FALSE zurckgeben
}


void CCustomizePageEditor::OnOK() 
{
	UpdateData();

	// get font size
	// CString	strFontSize;
	// m_wndEditorFontSize.GetLBText( m_nEditorFontSize, strFontSize );
	// g_configuration.m_nFontSize = (int)atol( strFontSize );

	// store settings
	StoreWindowTypeSettings();

	// copy color array
	for( int i = 0; i < CCrystalTextView::COLORINDEX_ERRORBKGND; i++ )
		g_configuration.m_aEditorColors[i] = m_aColors[i];

	// get tab width
	g_configuration.m_nTabWidth = (int)atol( m_strTabWidth );
	
	CPropertyPage::OnOK();
}


void CCustomizePageEditor::OnDestroy() 
{
	UpdateData();

	// get font size
	// CString	strFontSize;
	// m_wndEditorFontSize.GetLBText( m_nEditorFontSize, strFontSize );
	// g_configuration.m_nFontSize = (int)atol( strFontSize );

	// store settings
	StoreWindowTypeSettings();

	// copy color array
	for( int i = 0; i < CCrystalTextView::COLORINDEX_ERRORBKGND; i++ )
		g_configuration.m_aEditorColors[i] = m_aColors[i];

	// get tab width
	g_configuration.m_nTabWidth = (int)atol( m_strTabWidth );
	
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// update windows

	// this message will be send to all windows of the application
	AfxGetMainWnd()->SendMessage(WM_SYSCOLORCHANGE);

	CPropertyPage::OnDestroy();
}


void CCustomizePageEditor::OnSelchangeEditorElement() 
{
	UpdateData();

	// select element color
	m_wndElementColorPicker.EnableAutomaticButton(
		CString((LPCTSTR)STE_COLOR_AUTOMATIC), 
		CLatexEdit::GetAutomaticColor(m_wndEditorElement.GetItemData(m_nEditorElement)));
	m_wndElementColorPicker.SetColor(m_aColors[m_wndEditorElement.GetItemData(m_nEditorElement)]);
}


void CCustomizePageEditor::OnElementColor() 
{
	UpdateData();
	m_aColors[m_wndEditorElement.GetItemData(m_nEditorElement)] = m_wndElementColorPicker.GetColor();

}


BOOL CCustomizePageEditor::OnKillActive() 
{
	UpdateData();

	// check if options are valid
	if( atol( m_strTabWidth ) <= 0 || atol( m_strTabWidth ) > 20 )
	{
		AfxMessageBox( STE_EDITOROPTIONS_WRONGTABSIZE, MB_ICONINFORMATION | MB_OK );
		return FALSE;
	}
	
	return CPropertyPage::OnKillActive();
}


void CCustomizePageEditor::OnEditorSelectfont() 
{
	// select font for editor
	CFontDialog fontdlg;
	LOGFONT tmpFont;

	memcpy(&tmpFont, &m_logfont, sizeof(tmpFont));
	fontdlg.m_cf.lpLogFont = &tmpFont;
	fontdlg.m_cf.Flags|= (m_nWindowElement == wndEditor)? CF_INITTOLOGFONTSTRUCT|CF_FIXEDPITCHONLY : CF_INITTOLOGFONTSTRUCT;
	fontdlg.m_cf.Flags&= ~CF_EFFECTS;
	if (fontdlg.DoModal() == IDOK)
		fontdlg.GetCurrentFont(&m_logfont);

	UpdateFontExample();
}


void CCustomizePageEditor::OnSelchangeWindow() 
{
	StoreWindowTypeSettings();
	UpdateData();
	UseWindowTypeSettings();
	UpdateControlStates();
}
